# Copyright 2013-2014 Flavio Garcia
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from iflux.modules.admin.handlers import DashboardHandler
from iflux.core import ApplicationModule
import tornado.web
import os

class AdminModule(ApplicationModule):

    current_instance = None

    table_prefix = 'admin'
    connection_instance = 'admin'

    def get_handlers(self):
        handlers = [
            (r'/admin', DashboardHandler),
            (r"/admin/static/(.*)", tornado.web.StaticFileHandler,
             {"path": os.path.join(self.get_module_path(), 'static')}),
        ]
        return handlers

    def get_config_file(self):
        return "admin.ini"

    def process_config(self, config):
        if config.has_option('Admin', 'admin.table.prefix'):
            self.table_prefix = config.get('Admin', 'admin.table.prefix')
        if config.has_option('Admin', 'admin.data.connection.instance'):
            self.connection_instance = config.get('Admin',
                'admin.data.connection.instance'
            )
        AdminModule.current_instance = self

    def install(self):
        print "installing"
        import iflux.modules.admin.models
        from iflux.util.sqlalchemy_util import Base

        table_prefix = self.table_prefix
        engine = self.__get_connection_handler().get_connection()['engine']
        engine.echo=True
        # Dropping all
        Base.metadata.drop_all(engine)
        # Creating database
        Base.metadata.create_all(engine)

    def __get_connection_handler(self):
        return self.application.get_connection_handler(
            self.connection_instance
        )



# Copyright 2013-2014 Flavio Garcia
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import functools
import importlib
import types

def configure_connection(connection_instances, data_connected):
    if isinstance(connection_instances, types.StringTypes):
        import iflux.conf as iflux_conf
        # TODO Handler unknow connection instance here
        config = iflux_conf.data['connection']['instances'][connection_instances]
        connection_handler_config = iflux_conf.data['connection']['handlers'][config['type']]
        module = importlib.import_module(connection_handler_config['module'])
        handler_class = getattr(module, connection_handler_config['handler'])
        connection_handler = handler_class(data_connected)
        connection_handler.configure(config)
        data_connected.set_connection_handler(connection_instances, connection_handler)
        # Testing the connection
        # Without that the error will just happen during the handler execution
    elif isinstance(connection_instances, types.ListType):
        for connection_instance in connection_instances:
            configure_connection(connection_instance, data_connected)
    #TODO Throw an error here if it is not string or list

def configure(connection_instances):
    def f_wrapper(method):
        @functools.wraps(method)
        def wrapper(self, *args, **kwargs):
            configure_connection(connection_instances, self)
            return method(self, *args, **kwargs)
        return wrapper
    return f_wrapper


class DataConnectedMixin(object):
    """
    This mixin includes a configured session engine to an iflux application. The engine will be used by the session
    reads and writes triggered by the current handler.

    Example:
    >>> class MyClass(..., DataConnectedMixin):

    Refer to SessionEngine documentation in order to know which methods are available.
    """
    @property
    def connection_handlers(self):
        """Returns the data connections holded by the data connected object
        """
        return get_connection_handlers(self, '__connection_handlers')
    

    def get_connection_handler(self, name):
        """Returns a data connection instance
        """
        return self.connection_handlers[name]

    def set_connection_handler(self, name, connection_handler):
        self.connection_handlers[name] = connection_handler


class DataConnectedHolderMixin(object):

    @property
    def data_connected(self):
        return self.get_data_connected()

    def get_data_connected(self):
        return None


def get_connection_handlers(obj, connection_handlers_attribute):
    if not hasattr(obj, connection_handlers_attribute):
        setattr(obj, connection_handlers_attribute, {})
    return getattr(obj, connection_handlers_attribute)


def served_by(service, connection_name):

    def f_wrapper(method):
        @functools.wraps(method)
        def wrapper(self, *args, **kwargs):
            service_name = service.__name__
            first = True
            service_attribute = ''
            for s in service_name:
                if s.isupper():
                    if first:
                        service_attribute = ''.join([service_attribute, s.lower()])
                    else:
                        service_attribute = ''.join([service_attribute, '_', s.lower()])
                else:
                    service_attribute = ''.join([service_attribute, s])
                first = False
            setattr(self, service_attribute, service(self.data_connected.connection_handlers[connection_name]))
            return method(self, *args, **kwargs)

        return wrapper
    return f_wrapper



#
# Copyright 2013-2014 Flavio Garcia
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from itertools import islice, imap, repeat
import os
import string


class ConstantsHolder(object):

    class ConstantError(TypeError): pass

    def __setattr__(self,name,value):
        if self.__dict__.has_key(name):
            raise self.ConstantError, "Can't rebind constant(%s)"%name
        self.__dict__[name]=value


def random_string(length=5):
    """
    Generates a random string with the size equal to the given length.
    If length is not informed the string size will be 5.
    """
    chars = set(string.ascii_lowercase + string.digits)
    char_gen = (c for c in imap(os.urandom, repeat(1)) if c in chars)
    return ''.join(islice(char_gen, None, length))




# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import importlib
import os
import sys
import ConfigParser

# Setting root path
ROOT = ''
if os.path.abspath(__file__).endswith('.py') or \
        os.path.abspath(__file__).endswith('.pyc'):
    ROOT = os.path.dirname(__file__)
else:
    ROOT = os.path.abspath(__file__)

# Getting configuration paths and files from the environment
try:
    IFLUX_CONFIG_FILE = os.environ['IFLUX_CONFIG_FILE']
except KeyError:
    IFLUX_CONFIG_FILE = 'iflux.ini'

# Setting config files

#Library file
LIB_CONFIG_FILE = os.path.join(sys.modules[__name__].ROOT, 'conf',
                               IFLUX_CONFIG_FILE)

# Application file
APP_CONFIG_ROOT_PATH = os.path.join(os.getcwd())
APP_CONFIG_PATH = os.path.join(APP_CONFIG_ROOT_PATH, 'conf')
APP_CONFIG_FILE = os.path.join(APP_CONFIG_PATH, 'iflux.ini')


# Setting template path
TEMPLATE_PATH = os.path.join(sys.modules[__name__].ROOT, 'templates')

stack = []

HAS_LIB_CONFIG_FILE = False
HAS_APP_CONFIG_FILE = False

if os.path.isfile(LIB_CONFIG_FILE):
    HAS_LIB_CONFIG_FILE = True
    stack.append(LIB_CONFIG_FILE)

if os.path.isfile(APP_CONFIG_FILE):
    HAS_APP_CONFIG_FILE = True
    stack.append(APP_CONFIG_FILE)

# Setting iflux default variables

# Application default configuration
application = dict()

application['data'] = dict()
application['data']['connections'] = dict()

application['port'] = 8888

# Data default configuration
data = dict()

data['connection'] = dict()
data['connection']['instances'] = dict()
data['connection']['handlers'] = dict()

# Data default configuration
modules = dict()

# Session default configuration
session = dict()
session['enabled'] = False
session['type'] = 'file'
session['handlers'] = dict()
session['path'] = ''
session['name'] = 'IFLUXSESSID'
session['redis_configuration'] = {
    'connection': 'session',
}
session['redis_prefix'] = 'iflux:session'
session['encoding'] = dict()
session['encoding']['type'] = 'pickle'
session['encoding']['encoders'] = dict()

# Static assets default variables
static_assets = dict()
static_assets_location = ''
static_assets_directory = \
    os.path.join(sys.modules[__name__].ROOT, 'static_assets')


# Used to parse the framework(system?) config file
def parse_config(config):

    if config.has_section('Data'):
        parse_data_section(config)

    if config.has_section('Modules'):
        parse_modules_section(config)

    if config.has_section('Session'):
        parse_session_section(config)

    if config.has_section('Static_Assets'):
        parse_static_assets_section(config)


# Used to parse the application config file
def parse_application_config(config):
    parse_config(config)
    parse_application_section(config)


def parse_application_section(config):
    if config.has_option('Application', 'port'):
        sys.modules[__name__].application['port'] = \
            config.getint('Application', 'port')

    if config.has_option('Application', 'data.connections'):
        sys.modules[__name__].application['data']['connections'] = \
            [x.strip(' ') for x in config.get('Application',
                                              'data.connections').split(',')]


def parse_data_section(config):
    data_items = config.items('Data')
    
    # We should accumulate the handlers and instances
    connection_instances = \
        sys.modules[__name__].data['connection']['instances']

    for key, value in data_items:
        key_x = key.split('.')
        if 'data.connection.instance' in key:
            instance_name = key_x[-2]
            if instance_name not in connection_instances:
                connection_instances[instance_name] = dict()
            connection_instances[instance_name][key_x[-1]] = value
        if 'data.connection.handler' in key:
            value_x = value.split('.')
            type = key_x[-1]
            sys.modules[__name__].data['connection']['handlers'][type] = dict()
            sys.modules[__name__].data['connection']['handlers'][type][
                'module'] = '.'.join(value_x[:-1][:])
            # TODO change the conf key from handler ro class
            sys.modules[__name__].data['connection']['handlers'][type][
                'handler'] = value_x[-1]
    for key in connection_instances:
        # TODO if type is not here send an error
        type = connection_instances[key]['type']
        # TODO Warn that case here or handle it better
        module = importlib.import_module(sys.modules[__name__].data[
            'connection']['handlers'][type]['module'])
        db_config = getattr(module, sys.modules[__name__].data[
            'connection']['handlers'][type]['handler']).parse_config(
            connection_instances[key])
        sys.modules[__name__].data['connection']['instances'][key] = db_config


def parse_modules_section(config):
    modules_items = config.items('Modules')
    for key, value in modules_items:
        if 'modules.' in key:
            key_x = key.split('.')
            if len(key_x) > 1:
                if key_x[1] not in sys.modules[__name__].modules:
                    sys.modules[__name__].modules[key_x[1]] = dict()
                    sys.modules[__name__].modules[key_x[1]]['enabled'] = False
                if len(key_x) == 2:
                    value_x = value.split('.')
                    sys.modules[__name__].modules[key_x[1]]['class'] = \
                        value_x[-1]
                    sys.modules[__name__].modules[key_x[1]]['module'] = \
                        '.'.join(value_x[:-1][:])
                elif len(key_x) == 3:
                    if key_x[-1] in ['enabled']:
                        sys.modules[__name__].modules[key_x[1]][key_x[-1]] = \
                            config.getboolean('Modules', key)
                    else:
                        sys.modules[__name__].modules[key_x[1]][key_x[-1]] = \
                            value


def parse_session_section(config):
    if config.has_option('Session', 'session.enabled'):
        sys.modules[__name__].session['enabled'] = \
            config.getboolean('Session', 'session.enabled')

    if config.has_option('Session', 'session.type'):
        sys.modules[__name__].session['type'] = \
            config.get('Session', 'session.type')

    if sys.modules[__name__].session['type'] == 'file':
        if config.has_option('Session', 'session.path'):
            sys.modules[__name__].session['path'] = \
                config.get('Session', 'session.path')

    if sys.modules[__name__].session['type'] == 'redis':
        if config.has_option('Session', 'session.redis.host'):
            sys.modules[__name__].session['redis_configuration']['host'] = \
                config.get('Session', 'session.redis.host')
        if config.has_option('Session', 'session.redis.port'):
            sys.modules[__name__].session['redis_configuration']['port'] = \
                config.getint('Session', 'session.redis.port')
        if config.has_option('Session', 'session.redis.port'):
            sys.modules[__name__].session['redis_configuration']['db'] = \
                config.getint('Session', 'session.redis.db')

    if config.has_option('Session', 'session.encoding.type'):
        sys.modules[__name__].session['encoding']['type'] = \
            config.get('Session', 'session.encoding.type')

    session_items = config.items('Session')
    for key, value in session_items:
        if 'session.handler' in key:
            value_x = value.split('.')
            type = key.split('.')[-1]
            sys.modules[__name__].session['handlers'][type] = dict()
            sys.modules[__name__].session['handlers'][type]['module'] = \
                '.'.join(value_x[:-1][:])
            sys.modules[__name__].session['handlers'][type]['class'] = \
                value_x[-1]
        if 'session.encoding.encoder' in key:
            value_x = value.split('.')
            type = key.split('.')[-1]
            sys.modules[__name__].session['encoding']['encoders'][
                type] = dict()
            sys.modules[__name__].session['encoding']['encoders'][
                type]['module'] = '.'.join(value_x[:-1][:])
            sys.modules[__name__].session['encoding']['encoders'][
                type]['class'] = value_x[-1]

def parse_static_assets_section(config):
    if config.has_option('Static_Assets', 'static_assets.location'):
        sys.modules[__name__].static_assets_location = \
            config.get('Static_Assets', 'static_assets.location')

    static_assets_items = config.items('Static_Assets')
    for key, value in static_assets_items:
        if 'version' in key:
            key_x = key.split('.')
            sys.modules[__name__].static_assets[key_x[2]] = dict()
            sys.modules[__name__].static_assets[key_x[2]][key_x[3]] = value
            sys.modules[__name__].static_assets[key_x[2]]['name'] = key_x[2]


# TODO: hardcoded! Maybe a constant!?
if os.path.isfile('/etc/iflux/iflux.ini'):
    HAS_OS_CONFIG_FILE = True
else:
    HAS_OS_CONFIG_FILE = False

if HAS_LIB_CONFIG_FILE:
    config = ConfigParser.ConfigParser()
    config.readfp(open(LIB_CONFIG_FILE))
    parse_config(config)

if HAS_APP_CONFIG_FILE:
    config = ConfigParser.ConfigParser()
    config.readfp(open(APP_CONFIG_FILE))
    parse_application_config(config)



# Copyright 2013-2014 Flavio Garcia
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import httplib
import os
from urlparse import urlparse
import urllib2


def exists(url):
    """ Checks if an url exists.
    """
    host = urlparse(url).netloc
    conn = httplib.HTTPConnection(host)
    conn.request('HEAD', url)
    response = conn.getresponse()
    conn.close()
    return response.status == 200


def download_to_path(url, path_to):
    """ Downloads a given url to the a path.
    """
    parsed_url = urlparse(url)
    url_path = parsed_url.path
    file_name = url_path.split('/')[-1]
    
    download_file = os.path.join(path_to, file_name)

    response = urllib2.urlopen(url)

    # Open the file for wirting and reading it from request 
    # while writing it to the file
    with open(download_file, 'w') as f: f.write(response.read())



# Copyright 2013-2014 Flavio Garcia
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import iflux.conf
from iflux.util import file
import os


class FileSessionHandler(iflux.core.session.SessionHandler):
    """
    Session handler that deals with file data stored in files.

    The FileSessionHandler blocks tornado requests as long we are reading and
    writing files. We will try to minimize the blocking effect but by now don't
    use this for high traffic sites.
    """

    path = None

    def configure(self):
        if os.path.exists(iflux.conf.session['path']):
            self.path = iflux.conf.session['path']
        else:
            # TODO: Need to think about this. I don't know if this is a
            # good behaviour. Maybe just crash the app here
            # or disable the session with a good warning.
            self.path = '/tmp'

    def create_session(self, session_id, data):
        # TODO: What could possibly go wrong here? Let's handle it!
        session_file = os.path.join(self.path, self.__get_filename(session_id))
        if os.path.exists(iflux.conf.session['path']):
            file.touch(session_file)
            file.write(session_file, data)

    def read_stored_session(self, session_id):
        session_file = os.path.join(self.path, self.__get_filename(session_id))
        return file.read(session_file)

    def write_stored_session(self, session_id, data):
        session_file = os.path.join(self.path, self.__get_filename(session_id))
        file.write(session_file, data)

    def is_session_stored(self, session_id):
        return os.path.isfile(os.path.join(self.path, self.__get_filename(
            session_id)))

    def __get_filename(self, session_id):
        return 'iflux_%s.sess' % session_id


class RedisSessionHandler(iflux.core.session.SessionHandler):
    """
    Session handler that deals with file data stored  in a redis database.
    """

    connection_handler = None

    def configure(self):
        import redis
        configuration = iflux.conf.session['redis_configuration']
        self.connection_handler = self.engine.get_session_aware_instance().\
            get_connection_handler(configuration['connection'])

    def create_session(self, session_id, data):
        self.write_stored_session(session_id, data)

    def read_stored_session(self, session_id):
        return self.connection_handler.get_connection().get(
            self.__get_key(session_id))

    def write_stored_session(self, session_id, data):
        self.connection_handler.get_connection().set(
            self.__get_key(session_id), data)

    def is_session_stored(self, session_id):
        return self.connection_handler.get_connection().get(
            self.__get_key(session_id)) is not None

    def __get_key(self, session_id):
        return '%s:%s' % (iflux.conf.session['redis_prefix'], session_id)




# vim: set expandtab tabstop=4 shiftwidth=4 softtabstop=4:
#
# Copyright 2013-2014 Flavio Garcia
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import iflux.conf
import os
import sys

command_categories = dict()


def run_from_command_line():
    try:
        command = sys.argv[1]
        if not command_exists(command):
            show_command_line_usage()
        else:
            run_command(command)
    except IndexError:
        show_command_line_usage()

def show_command_line_usage():
    print "iFlux Tornado Framework"
    print "\nUsage: %s <command>" % os.path.split(sys.argv[0])[1]
    print "\nCommands are:\n"
    command_template = "  {0.name:10}{0.description:40}"
    for category, commands in command_categories.iteritems():
        print "%s:\n" % category
        for command in commands:
            print command_template.format(command)
    print ""
    print "    --name=value    whatever"
    print "    --help          display help"


def command_exists(command):
    for category, commands in command_categories.iteritems():
        for existing_command in commands:
            if command == existing_command.name:
                return True
    return False

def run_command(command):
    for category, commands in command_categories.iteritems():
        for existing_command in commands:
            if command == existing_command.name:
                print existing_command.tasks
                task = existing_command.tasks(existing_command)
                task.run()

class ScaffoldingCommand():

    def __init__(self, category, name, description, help, parameters=None, tasks=None):
        self.category = category
        self.name = name
        self.description = description
        self.help = help
        self.parameters = parameters
        self.tasks = tasks
        if category not in sys.modules[__name__].command_categories:
            sys.modules[__name__].command_categories[category] = []
        sys.modules[__name__].command_categories[category].append(self)

    def get_help(self):
        return self.help 


class ScaffoldingTask():

    def __init__(self, action):
        self.action = action

    def run(self):
        pass

    def get_help(self):
        pass


class CheckIfProjectExistsScaffoldingTask(ScaffoldingTask):

    def run(self):
        #TODO: Check if project exists
        print os.getcwd()


class CreateProjectScaffoldingTask(ScaffoldingTask):
    
    def run(self):
        #TODO: Check if project exists
        #TODO: If dosn't exists create project
        #TODO: If exists throw an error
        pass


class AddModuleToProjectScaffoldingTask(ScaffoldingTask):

    def run(self):
        #TODO: Check if module exists by name
        #TODO: If doesn't exists create module
        #TODO: If Exists throw an error
        pass


class RunProjectTask(ScaffoldingTask):

    def run(self):
        import tornado.ioloop
        import tornado.httpserver
        import iflux.conf
        import iflux.core

        http_server = tornado.httpserver.HTTPServer(iflux.core.Application())
        http_server.listen(iflux.conf.application['port'])
        tornado.ioloop.IOLoop.instance().start()

class InstallProjectTask(ScaffoldingTask):

    def run(self):
        import iflux.core
        application = iflux.core.Application()
        for key, module in application.modules.iteritems():
            module.install()

ScaffoldingCommand('iFlux', 'init', 'Create a new project in the given directory', '')

ScaffoldingCommand('iFlux', 'install', 'Install the application', '', [], InstallProjectTask)

ScaffoldingCommand('iFlux', 'run', 'Run the application', '', [], RunProjectTask)



# Copyright 2013-2014 Flavio Garcia
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import os
from tornado.template import Loader, Template


class ModuleLoader(Loader):
    """A template loader that loads from a single root directory.
    """
    module = None

    def __init__(self, root_directory, module=None, **kwargs):
        self.module = module
        super(ModuleLoader, self).__init__(root_directory, **kwargs)

    def resolve_path(self, name, parent_path=None):
        if ':' in name:
            nameX = name.split(':')
            module_name = nameX[0]
            name = os.path.join(self.module.application.modules[module_name].get_template_path(), nameX[-1])
        return super(ModuleLoader, self).resolve_path(name, parent_path)

    def _create_template(self, name):
        path = os.path.join(self.root, name)
        f = open(path, "rb")
        template = Template(f.read(), name=name, loader=self)
        f.close()
        return template




""" This code is based on:
http://code.activestate.com/recipes/577824-mixins-by-inheritance-vs-by-decoratorlets-try-deco/
"""


def add_mixin(cls, mixin, force=False):
    """Adds public attributes from a mixin to a class.
    """
    for name, value in mixin.__dict__.items():
        if name.startswith("_"):
            continue
        if not force and hasattr(cls, name):
            raise TypeError("name collision ({})".format(name))
        setattr(cls, name, value)
    try:
        mixin.register(cls)
    except AttributeError:
        pass


def mixes_in(*mixins, **parameters):
    """Decorator that receives a list of mixins and adds each mixin to the
    provided decorated class using add_mixin function.
    """
    def wrapper(cls):
        force = parameters.get('force') if parameters.has_key('force') \
            else False
        for mixin in mixins:
            add_mixin(cls, mixin, force)
        return cls
    return wrapper




# Copyright 2013-2014 Flavio Garcia
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from __future__ import absolute_import, division, print_function, with_statement

from iflux.core import session as isession
from iflux.core import template as itemplate
import iflux.core.data

from tornado.escape import json_encode
import tornado.web
import tornado


class RequestHandler(tornado.web.RequestHandler, iflux.core.data.DataConnectedHolderMixin):
    
    """ The Application Module that packs this request. This will be used
    to resolve the template path relative to the module."""
    module = None

    def initialize(self, application_module=None):
        if application_module is not None:
            self.module = application_module

    @isession.read
    def prepare(self):
        pass        

    @isession.write
    def on_finish(self):
        pass

    def render_string(self, template_name, **kwargs):
        ignore_module = False
        application_module = None
        for key in ('ignore_module', 'module',):
            if key in kwargs:
                if key is 'ignore_module':
                    ignore_module = kwargs[key]
                if key is 'module':
                   pass 
        kwargs['user_agent'] = self.user_agent if hasattr(self, 'user_agent') else None
        kwargs['credential'] = self.credential if hasattr(self, 'credential') else None
        return super(RequestHandler, self).render_string(template_name, **kwargs)

    def write(self, chunk):
        self.application.session
        if type(chunk) == JSONResponse:
            self.content_type = 'application/json'
            chunk = chunk.get_response()
        super(RequestHandler, self).write(chunk)

    def write_error(self, status_code, **kwargs):
        error_stack = {'code': status_code}

        exc_info = None
        for key in kwargs:
            if key == 'exc_info':
                exc_info = kwargs[key]
        error = exc_info[1]

        if type(error) == JSONError:
            error_stack.update(error.data)
            response = JSONResponse(data=None, error=error_stack)
            self.write(response)
        else:
            raise error

    def get_iflux_template_path(self):
        """Override to customize iflux template path for each handler.

        By default, we use the ``iflux_template_path`` application setting.
        Return None to load templates relative to the calling file.
        """
        return self.application.settings.get('iflux_template_path')

    def get_template_path(self):
        """Override to customize template path for each handler.

        By default, we use the ``template_path`` application setting.
        Return None to load templates relative to the calling file.
        """
        if self.module is None:
            # This is the default behaviour provided by Tornado.
            # No modules on the request no fancy template path.
            return super(RequestHandler, self).get_template_path()
        else:
            return self.module.get_template_path()

    def create_template_loader(self, template_path):
        """Returns a new template loader for the given path.

        May be overridden by subclasses.  By default returns a
        directory-based loader on the given path, using the
        ``autoescape`` application setting.  If a ``template_loader``
        application setting is supplied, uses that instead.
        """
        settings = self.application.settings
        kwargs = {}
        if "autoescape" in settings:
            # autoescape=None means "no escaping", so we have to be sure
            # to only pass this kwarg if the user asked for it.
            kwargs["autoescape"] = settings["autoescape"]
        return itemplate.ModuleLoader(template_path, module=self.module, **kwargs)

    def get_xsrf_token(self):
        return self.xsrf_token

    def get_data_connected(self):
        return self.application


class JSONResponse(object):

    def __init__(self, data={}, error=None):
        self.data = data
        self.error = error

    def get_response(self):
        return {'data': self.data, 'error': self.error}


class JSONError(tornado.web.HTTPError):

    data = {}

    def __init__(self, status_code, log_message=None, *args, **kwargs):
        self.data.update(log_message)
        if not isinstance(log_message, basestring):
            json_log_message = self.data
            json_log_message['code'] = status_code
            json_log_message = json_encode(json_log_message)
        super(JSONError, self).__init__(status_code, json_log_message, *args, **kwargs)



# Copyright 2013-2014 Flavio Garcia
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import hashlib

from sqlalchemy import CHAR, Column, SmallInteger, String
from sqlalchemy import Text, text, TIMESTAMP
from sqlalchemy.schema import Index, DefaultClause
from sqlalchemy.dialects.mysql.base import BIGINT, MEDIUMINT

import iflux.conf as iconf
from iflux.modules.admin import AdminModule
from iflux.util.sqlalchemy_util import Base

table_prefix = AdminModule.current_instance.table_prefix


class UserBase(Base):

    __tablename__ = '%s_user' % table_prefix

    mysql_engine='MyISAM'

    mysql_charset='utf8'

    # User id. This field is auto generated.
    id = Column('user_id', BIGINT(20, unsigned = True), primary_key=True)

    # User first name.
    first_name = Column('user_first_name', String(50),
                        DefaultClause(''), nullable=False)

    # User last name.
    last_name = Column('user_last_name', String(50),
                       DefaultClause(''), nullable=False)

    # User email address. This field is unique. This filed is used
    # to authenticate an user during the login process.
    email = Column('user_email', String(255),
                   DefaultClause(''), nullable=False, unique=True)

    # User password.
    password = Column('user_password', String(512),
                      DefaultClause(''), nullable=False)

    # User creation date.
    created = Column('created', TIMESTAMP,
                     DefaultClause('0000-00-00 00:00:00'), nullable=False)

    # User last update date
    updated = Column('updated', TIMESTAMP,
                     DefaultClause('0000-00-00 00:00:00'), nullable=False)


class GroupBase(Base):

    __tablename__ = 'group'

    mysql_engine='MyISAM'

    mysql_charset='utf8'

    # Group id. This field is auto generated.
    id = Column('group_id', BIGINT(20, unsigned=True), primary_key=True)

    # Group name.
    first_name = Column('group_name', String(50),
                        DefaultClause(''), nullable=False)

    # Group creation date.
    created = Column('created', TIMESTAMP,
                     DefaultClause('0000-00-00 00:00:00'), nullable=False)

    # Group last update date
    updated = Column('updated', TIMESTAMP,
                     DefaultClause('0000-00-00 00:00:00'), nullable=False)



# Copyright 2013-2014 Flavio Garcia
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import functools
import iflux.conf
import importlib
from iflux.util import random_string


# TODO Send to util!?
def get_class_from_conf(conf):
    module = importlib.import_module(conf['module'])
    return getattr(module, conf['class'])

class SessionEngine(object):
    """SessionEngine provides to an application session support.
    """

    session_handler = None
    session_encoder = None
    session_aware_instance = None

    def __init__(self, session_aware_instance):
        # TODO: By the way session could be disabled. How do we 
        # handle that?
        # TODO: check if session type exists. Maybe disable it if type is not
        # defined. We need to inform the error here
        handler_class = get_class_from_conf(
            iflux.conf.session['handlers'][iflux.conf.session['type']]
        ) 
        self.session_aware_instance = session_aware_instance
        self.session_handler = handler_class(self)
        self.session_handler.set_settings({})
        self.session_handler.configure()

        encoder_class = get_class_from_conf(
            iflux.conf.session['encoding']['encoders'][
                iflux.conf.session['encoding']['type']
            ]
        )
        self.session_encoder = encoder_class()

    def get_session(self, request_handler):
        """Returns a valid session object. This session is handler by the
        session handler defined on the application configuration.
        """
        session = Session()
        if iflux.conf.session['enabled']:
            cookie_created_on_request = False
            session_id = self.session_handler.get_session_id_cookie(
                request_handler)
            if session_id is None:
                cookie_created_on_request = True
                session_id = self.session_handler.create_session_id_cookie(
                    request_handler)
                self.session_handler.create_session(
                    session_id, self.encode_session_data(session.data))
            # TODO: Check if the session is stored
            if not self.session_handler.is_session_stored(session_id):
                if not cookie_created_on_request:
                    # Regenerating the session id. Because the
                    # cookie was not created during at the same request
                    # the file is being created.
                    session_id = self.session_handler.create_session_id_cookie(
                        request_handler)
                    self.session_handler.create_session(session_id,
                        self.encode_session_data(session.data))
            session_data = self.decode_session_data(
                self.session_handler.read_stored_session(session_id))
            session = Session(session_data)
            session.id = session_id
            return session

    def store_session(self, request_handler):
        """Sends the session data to be stored by the session handler defined
        on the application configuration.
        """
        if iflux.conf.session['enabled']:
            session_id = request_handler.session.id
            encoded_session_data = self.encode_session_data(
                request_handler.session.data)
            self.session_handler.write_stored_session(session_id,
                                                      encoded_session_data)

    def encode_session_data(self, data):
        return self.session_encoder.encode(data)

    def decode_session_data(self, data):
        return self.session_encoder.decode(data)

    def get_session_aware_instance(self):
        return self.session_aware_instance


class SessionEngineMixin(object):
    """
    This mixin includes a configured session engine to an iflux application.
    The engine will be used by the session reads and writes triggered by
    the current handler.

    Example:
    >>> class MyApplication(iflux.core.Application, SessionEngineMixin):

    Refer to SessionEngine documentation in order to know which methods are
    available.
    """

    @property
    def session_engine(self):
        """
        Returns a SessionManager instance
        """
        return create_session_engine(self, '__session_engine', SessionEngine)


class Session(object):

    id = None

    data = None

    def __init__(self, data={}):
        self.data = data

    def get(self, key):
        """
        Returns the value from the session data by a given key
        """
        if self.data.has_key(key):
            return self.data[key]
        return None

    def set(self, key, value):
        """
        Set a value into the session data labeled by a given key
        """
        self.data[key] = value


class SessionHandler(object):

    engine = None

    settings = None

    def __init__(self, engine):
        self.engine = engine
        self.settings = {}

    def create_session(self, session_id, data):
        pass

    def read_stored_session(self, session_id):
        pass

    def write_stored_session(self, session_id, data):
        pass

    def create_session_id_cookie(self, request_handler):
        session_id = self.__generate_session_id()
        request_handler.set_cookie(iflux.conf.session['name'], session_id,
                                   **self.__session_id_cookie_settings())
        return session_id

    def get_session_id_cookie(self, request_handler):
        return request_handler.get_cookie(iflux.conf.session['name'])

    def is_session_stored(self, session_id):
        pass

    def configure(self):
        pass

    def set_settings(self, settings):
        self.settings = settings

    def __session_id_cookie_settings(self):
        """
        Defines some settings to be used with the session id cookie
        """
        cookie_settings = {}
        cookie_settings.setdefault('expires', None)
        cookie_settings.setdefault('expires_days', None)
        return cookie_settings

    def __generate_session_id(self):
        # TODO we should make this generation more secure
        # TODO I think this is a good place to put a customized session
        # generator based on the configuration
        return random_string(64)


def create_session_engine(obj, session_engine_attribute, session_engine_class):
    if not hasattr(obj, session_engine_attribute):
        session_engine_instance = session_engine_class(obj)
        setattr(obj, session_engine_attribute, session_engine_instance)
    return getattr(obj, session_engine_attribute)


def read(method):
    @functools.wraps(method)
    def wrapper(self, *args, **kwargs):
        session = self.application.session_engine.get_session(self)
        self.session = session
        return method(self, *args, **kwargs)
    return wrapper


def write(method):
    @functools.wraps(method)
    def wrapper(self, *args, **kwargs):
        self.application.session_engine.store_session(self)
        return method(self, *args, **kwargs)
    return wrapper


class SessionEncoder(object):
    
    def encode(self, data):
        return data

    def decode(self, data):
        return data


class PickeSessionEncoder(SessionEncoder):

    def encode(self, data):
        import pickle
        return pickle.dumps(data, 2)

    def decode(self, data):
        import pickle
        return pickle.loads(data)



# Copyright 2013-2014 Flavio Garcia
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import functools
from tornado.web import HTTPError


# Security constants
DEFAULT_LOGIN_URL = '/login'
DEFAULT_LOGIN_PROCESS_URL = ''

try:
    import urlparse  # py2
except ImportError:
    import urllib.parse as urlparse  # py3

try:
    from urllib import urlencode  # py2
except ImportError:
    from urllib.parse import urlencode  # py3


class Credential(object):

    __authenticated = False

    def is_authenticated(self):
        return self.__authenticated

    def set_authenticated(self, authenticated):
        self.__authenticated = authenticated


class Secured(object):
    @property
    def credential(self):
        """
        Returns a SessionManager instance
        """
        return self.__get_credential(self, '__credential_manager', Credential)

    def __get_credential(self, obj, credential_attribute, credential_class):
        if not hasattr(obj, credential_attribute):
            setattr(obj, credential_attribute, credential_class())
        return getattr(obj, credential_attribute)

def secured(cls):
   for name, method in cls.__dict__.iteritems():
        if hasattr(method, "use_class"):
            # do something with the method and class
            print name, cls
   return cls


def identify(method):
    """
    Decorator that gets all security data from the session and adds to the credential.
    """
    @functools.wraps(method)
    def wrapper(self, *args, **kwargs):
        # TODO finish the session -> credential identification
        if self.session.get('UserId') is None:
            self.credential.set_authenticated(False)
        else:
            self.credential.set_authenticated(True)
        return method(self, *args, **kwargs)
    return wrapper


def authenticated(method):
    """
    Decorator that checks if the user is authenticated. If not send the user to the login page.
    """
    @functools.wraps(method)
    def wrapper(self, *args, **kwargs):
        is_authenticated = False
        if hasattr(self, 'credential'):
            if self.credition.is_authenticated():
                is_authenticated = True
        if not is_authenticated:
            if self.request.method in ("GET", "HEAD"):
                try:
                    url = self.get_login_url()
                except Exception:
                    # If url is not defined an exception will be raised
                    url = DEFAULT_LOGIN_URL
                if urlparse.urlsplit(url).scheme:
                    # if login url is absolute, make next absolute too
                    self.session.set('next_url', self.request.full_url())
                else:
                    self.session.set('next_url', self.request.uri)
                self.redirect(url)
                return
            raise HTTPError(403)
        return method(self, *args, **kwargs)
    return wrapper


def permissions(roles=[]):

    def f_wrapper(method):
        @functools.wraps(method)
        def wrapper(self, *args, **kwargs):
            print permissions
            print method
            return method(self, *args, **kwargs)

        return wrapper
    return f_wrapper



#!/usr/bin/env python
# vim: set expandtab tabstop=4 shiftwidth=4 softtabstop=4:
#
# Copyright 2013-2014 Flavio Garcia
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from distutils.core import setup
from setuptools import setup, find_packages
from distutils.command.build import build as _build

import iflux
import iflux.conf
from iflux.util import file
import os, sys
import subprocess


HG_LATEST_TAG_LABEL_CMD = 'hg log -r tip --template "{latesttag}.{latesttagdistance}"'
IFLUX_BUILD_FILE = os.path.join(os.path.dirname(__file__), 'build/lib/iflux/__init__.py')

def get_iflux_version():
    proc = subprocess.Popen(HG_LATEST_TAG_LABEL_CMD, shell=True,stdout=subprocess.PIPE)
    stdout_list = proc.communicate()[0].split('.')
    return stdout_list[-1]

def get_versioned_iflux_file_content():
    iflux_file_content = file.read(IFLUX_BUILD_FILE)
    return iflux_file_content.replace('"_REVISION_"', get_iflux_version())

class CustomBuildCommand(_build):
    def run(self):
        _build.run(self)
        dist_file = os.path.join(os.getcwd(), 'PKG-INFO')
        if not os.path.exists(dist_file):
            #TODO Add the static script routine here
            file.write(IFLUX_BUILD_FILE, get_versioned_iflux_file_content())

install_requires = [
        "tornado>=3.2",
        "redis>=2.9.1",
        "sqlalchemy==0.9.4",
    ] 

setup(
    name='iFlux',
    version= iflux.__version__.replace('_REVISION_', get_iflux_version()),
    description='iFlux is a python web framework based on Tornado web framework/server.',
    license='Apache License V2.0',
    author='Flavio Garcia',
    author_email='piraz@candango.org',
    install_requires=install_requires,
    cmdclass={
        'build': CustomBuildCommand,
    },
    url='http://www.candango.org/p/iflux',
    packages=[
        'iflux',
        'iflux.core', 
        'iflux.core.data',
        'iflux.core.scaffolding', 
        'iflux.core.session',
        'iflux.modules',
        'iflux.modules.admin',
        'iflux.modules.iflux',
        'iflux.util',
    ],
    package_dir={'iflux': 'iflux'},
    package_data={'iflux': ['conf/*.ini', 'modules/*/templates/*.html']},
    classifiers=[
        'Development Status :: 1 - Planning',
        'Environment :: Console',
        'Environment :: Web Environment',
        'License :: OSI Approved :: Apache Software License',
        'Operating System :: OS Independent',
        'Programming Language :: Python',
        'Programming Language :: Python :: 2',
        'Programming Language :: Python :: 2.7',
        'Topic :: Internet :: WWW/HTTP',
        'Topic :: Internet :: WWW/HTTP :: Dynamic Content',
        'Topic :: Internet :: WWW/HTTP :: WSGI',
        'Topic :: Software Development :: Libraries :: Application Frameworks',
        'Topic :: Software Development :: Libraries :: Python Modules',
    ],
    scripts=['iflux/bin/iflux-admin.py'],
    entry_points={'console_scripts': [
        'iflux = iflux.core.scaffolding:run_from_command_line',
    ]},
)




# Copyright 2013-2014 Flavio Garcia
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import sys


class ConnectionHandler(object):

    data_connected_instance = None

    def __init__(self, data_connected_instance):
        self.data_connected_instance = data_connected_instance


class RedisDataConnectionHandler(ConnectionHandler):

    connection = None
 
    def configure(self, config):
        import redis
        redis_config = dict()
        redis_config.update(config)
        redis_config.pop("type")
        # TODO Handle connection error
        self.connection = redis.Redis(**redis_config)

    def get_connection(self):
        return self.connection

    @staticmethod
    def parse_config(config):
        db_config = {
            'type': 'redis',
            'host': 'localhost',
            'port': 6379,
            'db': 0,
        }
        for key in config:
            if key in ['db', 'host', 'port']:
                if key in ['db', 'port']:
                    db_config[key] = int(config[key])
                db_config[key] = config[key]
        return db_config


class SqlalchemyDataConnectionHandler(ConnectionHandler):
    
    __connection = {
        'engine': None,
        'session': None,
    }
   
    def configure(self, config):
        from sqlalchemy import create_engine
        from sqlalchemy.exc import OperationalError
        from iflux.util.sqlalchemy_util import Session

        self.__connection['engine'] = create_engine(config['url'])
        try:
            self.__connection['engine'].connect()
        except OperationalError as error:
            sys.exit("Error connecting to the database: " + error.message)

        Session.configure(bind=self.__connection['engine'])
        self.__connection['session'] = Session()
        # TODO: Test the session right here. Without that the error 
        # will just happen during the handler execution

    def get_connection(self):
        return self.__connection

    @property
    def engine(self):
        return self.__connection['engine']

    @property
    def session(self):
        return self.__connection['session']

    @staticmethod
    def parse_config(config):
        db_config = {
            'type': 'sqlalchemy',
        }
        for key in config:
            # TODO Need to handle other properies and create the url if needed.
            if key in ['url']:
                db_config[key] = config[key]
        return db_config



__author__ = 'fpiraz'
import functools
import sys

import sqlalchemy
from sqlalchemy import create_engine
from sqlalchemy.exc import OperationalError
from sqlalchemy.ext.declarative import declarative_base

Base = declarative_base()
Session = sessionmaker()

class SqlalchemyEnabled():

    @property
    def sqlalchemy_engine(self):
        """
        Returns a Sqlalchemy engine instance
        """
        return getattr(self, '__slqalchemy_engine')

    @property
    def sqlalchemy_session(self):
        """
        Returns a Sqlalchemy Session instance
        """
        return getattr(self, '__slqalchemy_session')




# vim: set expandtab tabstop=4 shiftwidth=4 softtabstop=4:
#
# Copyright 2013-2014 Flavio Garcia
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import ConfigParser
import iflux.conf
from iflux.core.data import configure_connection, DataConnectedMixin
from iflux.core.session import SessionEngineMixin
import importlib
import inspect
import os
import tornado.web


class Application(tornado.web.Application, DataConnectedMixin,
                  SessionEngineMixin):

    session = None

    modules = {} 

    def __init__(self, informed_handlers=[], **settings):
        configure_connection(iflux.conf.application['data']['connections'], self)
        static_handlers = [
            (r"/static_assets/(.*)", tornado.web.StaticFileHandler,
             {"path": iflux.conf.static_assets_directory}),
        ]

        if 'iflux_template_path' not in settings:
            settings['iflux_template_path'] = iflux.conf.TEMPLATE_PATH

        # TODO: Make shure they don't specify reserved handlers here
        handlers = informed_handlers + static_handlers
        
        self.__load_modules()

        for key, module in self.modules.iteritems():
            module_handlers = module.get_handlers()
            for i in range(0, len(module_handlers)):
                if len(module_handlers[i]) < 3:
                    module_handlers[i] = (module_handlers[i][0],
                                          module_handlers[i][1],
                                          {'application_module': module})
            handlers = handlers + module_handlers
        tornado.web.Application.__init__(self, handlers, **settings) 

    def __load_modules(self):
        for key, value in iflux.conf.modules.iteritems():
            if value['enabled']:
                module_module = importlib.import_module(value['module'])
                module_class = getattr(module_module, value['class'])
                self.modules[key] = module_class(key, self)
                module_config_file = os.path.join(
                    iflux.conf.APP_CONFIG_PATH,
                    self.modules[key].get_config_file())
                config = ConfigParser.ConfigParser()
                if os.path.isfile(module_config_file):
                    config.readfp(open(os.path.join(module_config_file)))
                self.modules[key].process_config(config)


class ApplicationModule(object):


    def __init__(self, name, application):
        self.name = name
        self.application = application
        self.configuration = dict()

    def get_handlers(self):
        return []

    def get_module_path(self):
        return os.path.dirname(inspect.getfile(self.__class__))

    def get_template_path(self):
        return self.__build_template_path()

    def __build_template_path(self):
        return os.path.join(os.path.dirname(
            inspect.getfile(self.__class__)), 'templates')

    def get_config_file(self):
        return 'iflux.ini'

    def process_config(self, config):
        pass

    def pre_install(self):
        pass

    def install(self):
        pass

    def post_install(self):
        pass
